import eq_read_f
import numpy as np
import matplotlib
from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.tri as mtri
import os
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from matplotlib.backend_bases import key_press_handler
from tkinter.filedialog import askopenfilename


uni_font = ("Helvetica", 20)
small_font = ("Helvetica", 14)
plt.rcParams.update({'font.size': 14, 'font.family': 'serif'})


class ToplevelData(tk.Toplevel):
    def __init__(self, master=None, cnf={}, **kw):
        super(ToplevelData, self).__init__(master=master, cnf=cnf, **kw)
        self.read = True

    def destroy(self):
        eq_read_f.eq_parameters.deallocate()
        super(ToplevelData, self).destroy()
        self.read = False


def read(directory):
    eq_file_name = os.path.join(directory, 'equilibrium.out')
    eq_read_f.eq_parameters.filename(eq_file_name)
    eq_read_f.eq_parameters.eq_read()
    print("lsr: %d, lsz %d" %
          (eq_read_f.eq_parameters.lsr, eq_read_f.eq_parameters.lsz))


def panel(window, global_win_grp, work_dir):
    # p = tk.Toplevel(window, width=300, height=200)
    p = ToplevelData(window, width=300, height=200)
    global_win_grp.append(p)
    p.protocol("WM_DELETE_WINDOW", p.destroy)
    p.title("equilibrium")
    read(work_dir)

    # rgrid
    r_ticks = np.arange(1, eq_read_f.eq_parameters.lsr+1)
    r_grids = eq_read_f.eq_parameters.r_eq
    b_r = tk.Button(p,
                    text='R',
                    command=lambda: plot_eq_1d(window,
                                               "eq R grids",
                                               r_ticks,
                                               r_grids,
                                               "grids #"),
                    font=uni_font)
    b_r.grid(row=1, column=1, sticky=tk.W+tk.E)
    # zgrid
    z_ticks = np.arange(1, eq_read_f.eq_parameters.lsz+1)
    z_grids = eq_read_f.eq_parameters.z_eq
    b_z = tk.Button(p,
                    text='Z',
                    command=lambda: plot_eq_1d(window,
                                               "eq Z grids",
                                               z_ticks,
                                               z_grids,
                                               "grids #"),
                    font=uni_font)
    b_z.grid(row=2, column=1, sticky=tk.W+tk.E)
    # ni
    ni = eq_read_f.eq_parameters.ni_eq
    b_ni = tk.Button(p, text='ni',
                     command=lambda: plot_eq_2d(window, "$n_i$", ni),
                     font=uni_font)
    b_ni.grid(row=1, column=2, sticky=tk.W+tk.E)
    # ne
    ne = eq_read_f.eq_parameters.ne_eq
    b_ne = tk.Button(p, text='ne',
                     command=lambda: plot_eq_2d(window, "$n_e$", ne),
                     font=uni_font)
    b_ne.grid(row=2, column=2, sticky=tk.W+tk.E)
    # ti
    ti = eq_read_f.eq_parameters.ti_eq
    b_ti = tk.Button(p, text='Ti',
                     command=lambda: plot_eq_2d(window, "$T_i$", ti),
                     font=uni_font)
    b_ti.grid(row=3, column=2, sticky=tk.W+tk.E)
    # te
    te = eq_read_f.eq_parameters.te_eq
    b_te = tk.Button(p, text='Te',
                     command=lambda: plot_eq_2d(window, "$T_e$", te),
                     font=uni_font)
    b_te.grid(row=4, column=2, sticky=tk.W+tk.E)
    # psi
    psi = eq_read_f.eq_parameters.psirz_eq
    b_psi = tk.Button(p, text='psi',
                      command=lambda: plot_eq_2d(window, "$\psi$", psi),
                      font=uni_font)
    b_psi.grid(row=1, column=3, sticky=tk.W+tk.E)
    # B0
    b0 = eq_read_f.eq_parameters.b0mag_eq
    b_b0 = tk.Button(p, text='B0',
                     command=lambda: plot_eq_2d(window, "$B$", b0),
                     font=uni_font)
    b_b0.grid(row=3, column=3, sticky=tk.W+tk.E)
    # BR
    br = eq_read_f.eq_parameters.b0r_eq
    b_br = tk.Button(p, text='Br',
                     command=lambda: plot_eq_2d(window, "$B_R$", br),
                     font=uni_font)
    b_br.grid(row=4, column=3, sticky=tk.W+tk.E)
    # BZ
    bz = eq_read_f.eq_parameters.b0z_eq
    b_bz = tk.Button(p, text='Bz',
                     command=lambda: plot_eq_2d(window, "$B_z$", bz),
                     font=uni_font)
    b_bz.grid(row=5, column=3, sticky=tk.W+tk.E)


def plot_eq_1d(window, title, x_ticks, values, xname):
    new_win = tk.Toplevel(window)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(x_ticks, values, 'ro-', mfc='none')
    ax.set_title(title)
    ax.set_xlabel(xname, fontsize=14)
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def plot_eq_2d(window, title, values):
    new_win = tk.Toplevel(window)
    rgrids = eq_read_f.eq_parameters.r_eq
    zgrids = eq_read_f.eq_parameters.z_eq
    Z, R = np.meshgrid(zgrids, rgrids)
    fig = plt.figure()

    ax = fig.add_subplot(111)
    tpc = ax.pcolormesh(Z, R, values)
    cmap = tpc.get_cmap()
    cmap.set_over("white")
    cmap.set_under("white")
    ax.set_title(title)
    ax.set_xlabel("Z[m]", fontsize=14)
    ax.set_ylabel("R[m]", fontsize=14)
    cbar = fig.colorbar(tpc)
    cbar.formatter.set_powerlimits((-1, 1))
    cbar.update_ticks()
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)
    b_color = tk.Button(new_win, text='color',
                        command=lambda: reclim(new_win, tpc, canvas),
                        font=small_font)
    b_color.pack(side="left", expand=False)
    b_contour = tk.Button(new_win, text='get Contour',
                          command=lambda: contour(window, title, tpc, values),
                          font=small_font)
    b_contour.pack(side="left", expand=False)
    equal = tk.BooleanVar()
    equal.set(False)
    b_change = tk.Button(new_win, text='aixs aspect Equal/Auto',
                         command=lambda: aspect(ax, canvas, equal),
                         font=small_font)
    b_change.pack(side="left", expand=False)
    is_cbar = tk.BooleanVar()
    is_cbar.set(True)
    b_cbar = tk.Button(new_win, text='remove/add cbar',
                       command=lambda: switch_cbar(is_cbar, tpc, canvas, fig),
                       font=small_font)
    b_cbar.pack(side="left", expand=True)


def switch_cbar(is_cbar, tpc, canvas, fig):
    ax = fig.axes[1]
    if is_cbar.get():
        ax.set_visible(False)
        is_cbar.set(False)
    else:
        ax.set_visible(True)
        is_cbar.set(True)
    fig.tight_layout()
    canvas.draw()


def aspect(ax, canvas, equal):
    if equal.get():
        ax.set_aspect("auto")
        equal.set(False)
    else:
        ax.set_aspect("equal")
        equal.set(True)
    canvas.draw()


def contour(window, title, tpc, values):
    contour_win = tk.Toplevel(window)
    rgrids = eq_read_f.eq_parameters.r_eq
    zgrids = eq_read_f.eq_parameters.z_eq
    Z, R = np.meshgrid(zgrids, rgrids)
    fig = plt.figure()

    ax = fig.add_subplot(111)
    vmin, vmax = tpc.get_clim()
    tpc = ax.contour(Z, R, values, np.linspace(vmin, vmax, 20))
    cmap = tpc.get_cmap()
    cmap.set_over("white")
    cmap.set_under("white")
    ax.set_title(title)
    ax.set_xlabel("Z[m]", fontsize=14)
    ax.set_ylabel("R[m]", fontsize=14)
    cbar = fig.colorbar(tpc)
    cbar.formatter.set_powerlimits((-1, 1))
    cbar.update_ticks()
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=contour_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, contour_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)
    equal = tk.BooleanVar()
    equal.set(False)
    b_change = tk.Button(contour_win, text='aixs aspect Equal/Auto',
                         command=lambda: aspect(ax, canvas, equal),
                         font=uni_font)
    b_change.pack(side="left", expand=False)


def reclim(root, tpc, canvas):
    crt_clim = tpc.get_clim()
    pop = tk.Toplevel(root)
    # info = "Reset max and min value for color"
    l1 = tk.Label(pop, text="vmin")
    l2 = tk.Label(pop, text="vmax")
    vmin_text = tk.DoubleVar()
    vmax_text = tk.DoubleVar()
    vmin_text.set(crt_clim[0])
    vmax_text.set(crt_clim[1])
    min_entry = tk.Entry(pop, textvariable=vmin_text)
    max_entry = tk.Entry(pop, textvariable=vmax_text)
    b_set = tk.Button(pop, text='Set', command=lambda: set_clim(canvas,
                                                                pop, tpc, vmin=vmin_text.get(), vmax=vmax_text.get()))
    array = tpc.get_array()
    minv = array.min()
    maxv = array.max()
    b_reset = tk.Button(pop, text="Reset", command=lambda: set_clim(canvas,
                                                                    pop, tpc, vmin=minv, vmax=maxv))
    l1.grid(row=1, column=1)
    l2.grid(row=2, column=1)
    min_entry.grid(row=1, column=2, columnspan=2)
    max_entry.grid(row=2, column=2, columnspan=2)
    b_set.grid(row=3, column=1)
    b_reset.grid(row=3, column=2)


def set_clim(canvas, pop, tpc, vmin, vmax):
    tpc.set_clim(vmin=vmin, vmax=vmax)
    canvas.draw()
    pop.destroy()
