import tkinter as tk
import grids_read_f
import grids_eq_read_f
from matplotlib import pyplot as plt
import numpy as np
import os
import matplotlib.tri as mtri
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
import matplotlib.transforms as mtransforms

numerfrc = 3
uni_font = ("Helvetica", 20)
small_font = ("Helvetica", 14)

def mask_func(x, y, limit):
    distance0 = np.sqrt((x[:, 1]-x[:, 0])**2+(y[:, 1]-y[:, 0])**2)
    distance1 = np.sqrt((x[:, 2]-x[:, 1])**2+(y[:, 2]-y[:, 1])**2)
    distance2 = np.sqrt((x[:, 0]-x[:, 2])**2+(y[:, 0]-y[:, 2])**2)

    return np.max(np.array([distance0, distance1, distance2]), axis=0) > limit


class ToplevelData(tk.Toplevel):
    def __init__(self, master=None, cnf={}, **kw):
        super(ToplevelData, self).__init__(master=master, cnf=cnf, **kw)
        self.read = True

    def destroy(self):
        grids_eq_read_f.grids_eq_parameters.deallocate()

        if grids_read_f.grids_parameters.allocated:
            grids_read_f.grids_parameters.deallocate()
            grids_read_f.grids_parameters.allocated = False

        super(ToplevelData, self).destroy()
        self.read = False


def read(work_dir):
    grids_eq_name = os.path.join(work_dir, 'grids_eq.out')
    grids_name = os.path.join(work_dir, 'grids.out')
    grids_eq_read_f.grids_eq_parameters.filename(grids_eq_name)
    grids_eq_read_f.grids_eq_parameters.numerfrc = numerfrc
    grids_eq_read_f.grids_eq_parameters.read()

    if not grids_read_f.grids_parameters.allocated:
        grids_read_f.grids_parameters.filename(grids_name)
        grids_read_f.grids_parameters.numerfrc = numerfrc
        grids_read_f.grids_parameters.grids_read()
        grids_eq_read_f.grids_eq_parameters.allocated = True


def panel(window, global_win_grp, work_dir):
    # p = tk.Toplevel(window,width=300, height=200)
    p = ToplevelData(window, width=300, height=200)
    p.title("grids_eq")
    global_win_grp.append(p)
    read(work_dir)

    b_grids = tk.Button(p, text="Grids",
                        command=lambda: plot_grids(window),
                        font=uni_font)
    b_grids.grid(row=1, column=1, sticky=tk.W+tk.E)
    rgrids = grids_read_f.grids_parameters.rg1d
    psigrids = grids_eq_read_f.grids_eq_parameters.psimesh
    b_psi = tk.Button(p, text="Psi(r)",
                      command=lambda: plot1d(
                          window, rgrids, psigrids, "R", "$\psi$"),
                      font=uni_font)
    b_psi.grid(row=2, column=1, sticky=tk.W+tk.E)

    kti = grids_eq_read_f.grids_eq_parameters.kapatip
    b_kti = tk.Button(p, text="kappati",
                      command=lambda: plot1d(
                          window, rgrids, kti, "R", "$\kappa_{Ti}$"),
                      font=uni_font)
    b_kti.grid(row=1, column=2, sticky=tk.W+tk.E)

    kni = grids_eq_read_f.grids_eq_parameters.kapanip
    b_kni = tk.Button(p, text="kappani",
                      command=lambda: plot1d(
                          window, rgrids, kni, "R", "$\kappa_{ni}$"),
                      font=uni_font)
    b_kni.grid(row=2, column=2, sticky=tk.W+tk.E)

    kte = grids_eq_read_f.grids_eq_parameters.kapatep
    b_kte = tk.Button(p, text="kappate",
                      command=lambda: plot1d(
                          window, rgrids, kte, "R", "$\kappa_{Te}$"),
                      font=uni_font)
    b_kte.grid(row=3, column=2, sticky=tk.W+tk.E)

    kne = grids_eq_read_f.grids_eq_parameters.kapanep
    b_kne = tk.Button(p, text="kappane",
                      command=lambda: plot1d(
                          window, rgrids, kne, "R", "$\kappa_{ne}$"),
                      font=uni_font)
    b_kne.grid(row=4, column=2, sticky=tk.W+tk.E)

    b_psip = tk.Button(p, text='psi prime',
        command=lambda:psi_prime(window, rgrids, psigrids),
        font=uni_font)
    b_psip.grid(row=3,column=1,sticky=tk.W+tk.E)

    ni = grids_eq_read_f.grids_eq_parameters.meshni
    b_ni = tk.Button(p, text="ni",
                      command=lambda: flattern(
                          window, rgrids, ni, "R", "$n_i$"),
                      font=uni_font)
    b_ni.grid(row=1, column=3, sticky=tk.W+tk.E)

    ne = grids_eq_read_f.grids_eq_parameters.meshne
    b_ne = tk.Button(p, text="ne",
                      command=lambda: flattern(
                          window, rgrids, ne, "R", "$n_e$"),
                      font=uni_font)
    b_ne.grid(row=2, column=3, sticky=tk.W+tk.E)

    ti = grids_eq_read_f.grids_eq_parameters.meshti
    b_ti = tk.Button(p, text="Ti",
                      command=lambda: flattern(
                          window, rgrids, ti, "R", "$T_i$"),
                      font=uni_font)
    b_ti.grid(row=3, column=3, sticky=tk.W+tk.E)

    te = grids_eq_read_f.grids_eq_parameters.meshte
    b_te = tk.Button(p, text="Te",
                      command=lambda: flattern(
                          window, rgrids, te, "R", "$T_e$"),
                      font=uni_font)
    b_te.grid(row=4, column=3, sticky=tk.W+tk.E)

    bsp = grids_eq_read_f.grids_eq_parameters.bmesh
    b_bsp = tk.Button(p, text="bsp",
                      command=lambda: flattern(
                          window, rgrids, bsp, "R", "$bsp$"),
                      font=uni_font)
    b_bsp.grid(row=1, column=4, sticky=tk.W+tk.E)


def flattern(window, x, y, xlabel, ylabel):
    # if numerfrc == 3:
    #     r_all = grids_read_f.grids_parameters.rsol
    #     z_all = grids_read_f.grids_parameters.zsol
    # else:
    #     raise Exception
    grids_r = x.shape[0]
    # mpsi_sol = grids_eq_read_f.grids_eq_parameters.mpsi_sol
    # mpara_sol = grids_eq_read_f.grids_eq_parameters.mpara_sol
    # r_2d = r_all.reshape((mpsi_sol, mpara_sol))
    # z_2d = z_all.reshape((mpsi_sol, mpara_sol))
    y2d = y.reshape((grids_r, -1))

    # win_2d = tk.Toplevel(window)
    # fig2d = plt.figure()
    # ax2d = fig2d.add_subplot(111)
    # tpc=ax2d.pcolormesh(r_2d,z_2d, y2d)
    # ax2d.set_xlabel(xlabel)
    # ax2d.set_ylabel(ylabel)
    # fig2d.tight_layout()
    # fig2d.colorbar(tpc)
    # canvas2d = FigureCanvasTkAgg(fig2d, master=win_2d)
    # canvas2d.draw()

    # toolbar2d = NavigationToolbar2Tk(canvas2d, win_2d)
    # toolbar2d.update()
    # canvas2d.get_tk_widget().pack(side="top", fill='both', expand=True)

    grids_z = y2d.shape[1]
    y1d = (y2d[:, grids_z//2] + y2d[:, (grids_z-1)//2])/2
    new_win = tk.Toplevel(window)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    tpc = ax.plot(x[:-2], y1d[:-2], 'o-')
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)

def psi_prime(window, rgrids, psigrids):
    new_win = tk.Toplevel(window)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    length = len(rgrids)
    prime = np.zeros(length-2)
    for i in range(length-2):
        pc = psigrids[i+1]
        rc = rgrids[i+1]
        pl = psigrids[i]
        rl = rgrids[i]
        pr = psigrids[i+2]
        rr = rgrids[i+2]
        prime[i] = ((pc-pl)/(rc-rl))/(rr-rl)*(rr-rc)+((pr-pc)/(rr-rc))/(rr-rl)*(rc-rl)
    ax.plot(rgrids[1:-1], prime, 'ro-')
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def plot1d(window, x, y, xlabel, ylabel):
    new_win = tk.Toplevel(window)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    tpc = ax.plot(x, y, 'o-')
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def plot_grids(window):
    if numerfrc == 3:
        r_all = grids_read_f.grids_parameters.rsol
        z_all = grids_read_f.grids_parameters.zsol
    else:
        raise Exception

    mpsi_sol = grids_eq_read_f.grids_eq_parameters.mpsi_sol
    mpara_sol = grids_eq_read_f.grids_eq_parameters.mpara_sol
    r_2d = r_all.reshape((mpsi_sol, mpara_sol))
    z_2d = z_all.reshape((mpsi_sol, mpara_sol))

    new_win = tk.Toplevel(window)
    fm1 = tk.Frame(new_win)
    
    fig = plt.figure()
    ax = fig.add_subplot(111)

    for i in range(mpsi_sol):
        ax.plot(r_2d[i], z_2d[i], 'k-')

    ax.plot(r_all, z_all, 'rx')
    ax.set_xlabel('R')
    ax.set_ylabel('Z')

    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    

    equal = tk.BooleanVar()
    equal.set(False)
    b_change = tk.Button(fm1, text='aixs aspect Equal/Auto',
                         command=lambda: aspect(ax, canvas, fig, equal),
                         font=small_font)

    rot = tk.BooleanVar()
    rot.set(False)
    b_rot = tk.Button(fm1, text='rotate',
                      command=lambda: rotate(
                          ax, canvas, fig, rot),
                      font=small_font)

    is_all = tk.BooleanVar()
    is_all.set(True)
    b_switchg = tk.Button(fm1, text='All/sample',
                          command=lambda: switch_grids(
                              r_2d, z_2d, ax, canvas, is_all, rot),
                          font=small_font)
    b_redraw = tk.Button(fm1, text='Redraw',
    command=lambda: redraw(fig, canvas),
    font=small_font)
    b_switchg.pack(side="left", expand=False)
    b_change.pack(side="left", expand=False)
    b_rot.pack(side="left", expand=False)
    b_redraw.pack(side="left", expand=False)
    fm1.pack(side="top",fill="both",expand=True)
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)

def redraw(fig, canvas):
    fig.tight_layout()
    canvas.draw()

def rotate(ax, canvas, fig, rot):
    base = ax.transData
    rotTrans = mtransforms.Affine2D().rotate_deg(90)
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    xlabel = ax.get_xlabel()
    ylabel = ax.get_ylabel()
    if rot.get():
        rot.set(False)
        for c in ax.get_lines():
            c.set_transform(base)
    else:
        rot.set(True)
        for c in ax.get_lines():
            c.set_transform(rotTrans + base)
    ax.set_xlim(*ylim)
    ax.set_ylim(*xlim)
    ax.set_xlabel(ylabel)
    ax.set_ylabel(xlabel)
    fig.tight_layout()
    canvas.draw()


def switch_grids(r_2d, z_2d, ax, canvas, is_all, rot):
    shape = r_2d.shape
    ind0_samp = np.linspace(0, shape[0]-1, 6, dtype=int)
    ind1_samp = np.linspace(0, shape[1]-1, 20, dtype=int)
    sampled_r = r_2d[ind0_samp, :][:, ind1_samp]
    sampled_z = z_2d[ind0_samp, :][:, ind1_samp]
    ax.clear()
    rot.set(False)
    if is_all.get():
        is_all.set(False)
        for i in ind0_samp:
            ax.plot(r_2d[i], z_2d[i], 'k-')
        ax.plot(sampled_r, sampled_z, 'rx')
    else:
        is_all.set(True)
        for i in range(shape[0]):
            ax.plot(r_2d[i], z_2d[i], 'k-')
        ax.plot(r_2d, z_2d, 'rx')
    ax.set_xlabel('R')
    ax.set_ylabel('Z')
    canvas.draw()


def aspect(ax, canvas, fig, equal):
    if equal.get():
        ax.set_aspect("auto")
        equal.set(False)
    else:
        ax.set_aspect("equal")
        equal.set(True)
    fig.tight_layout()
    canvas.draw()


def main(dir_path="."):
    path = os.path.abspath(dir_path)
    read(path)
    r_all = grids_read_f.grids_parameters.rsol
    z_all = grids_read_f.grids_parameters.zsol
    global triang
    triang = mtri.Triangulation(r_all, z_all)
    triang.set_mask(
        mask_func(r_all[triang.triangles], z_all[triang.triangles], 0.5))
    psimesh = grids_eq_read_f.grids_eq_parameters.psimesh_sol
    kti = grids_eq_read_f.grids_eq_parameters.kapatip
    kni = grids_eq_read_f.grids_eq_parameters.kapanip
    kte = grids_eq_read_f.grids_eq_parameters.kapatip
    kne = grids_eq_read_f.grids_eq_parameters.kapanip
    plt.plot(psimesh, kni)
    plt.show()


if __name__ == "__main__":
    main()
