import numpy as np
import matplotlib
from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import argparse
import os
import re

def index_mapping(data_array):
    particle_nums_in_file = np.array(re.split(r'\s+', data_array[0][1].strip()), dtype=int)
    index_map = [{} for _ in particle_nums_in_file]
    zdims = [0 for _ in particle_nums_in_file]
    invert_map = {}
    global_part_num = 0
    file_num = len(data_array)
    for file_ind in range(file_num):
        #skip first line indicating the step
        line = 1
        line1 = data_array[file_ind][line]
        particle_nums_in_file = np.array(re.split(r'\s+', line1.strip()), dtype=int)
        for sp_ind, one_species_num in enumerate(particle_nums_in_file):
            for p in range(one_species_num):
                line += 1
                zpart = np.array(re.split(r'\s+', data_array[file_ind][line].strip()), dtype=float)
                zdims[sp_ind]=len(zpart)
                local_index = (int(zpart[-2]), int(zpart[-1]))
                global_part_num += 1
                index_map[sp_ind][local_index] = global_part_num - 1
                invert_map[global_part_num - 1]=(sp_ind, int(zpart[-2]), int(zpart[-1]))

    
    return zdims, index_map, global_part_num, invert_map


def pre_read(dirname):
    files = os.listdir(dirname)
    data_array = []
    fname_array= []
    for fname in files:
        if fname.startswith('TRACKP'):
            with open(os.path.join(dirname, fname)) as f:
                fname_array.append(fname)
                data_array.append(re.split(r'\n+', f.read())[:-1])

    line = 0
    step = 0
    
    print(len(data_array), len(data_array[0]))
    while line < len(data_array[0]):
        step += 1
        line += 1
        part_nums = np.array(re.split(r'\s+', data_array[0][line].strip()), dtype=int).sum()
        line += part_nums + 1
        
    return fname_array, data_array, step

def read(fname_array, data_array, step):
    zdims, index_map, global_part_num, invert_map = index_mapping(data_array)
    species = len(index_map)
    orbit_array = np.zeros(shape=(global_part_num,zdims[0],step))
    line_indicator = [0 for _ in fname_array]
    step_indicator = [0 for _ in range(global_part_num)]
    reading = True
    while reading:
        for (ind, num) in enumerate(line_indicator):
            #skip `istep` in tracking files and read # of tracked particles
            line_indicator[ind] += 1
            if line_indicator[ind] >= len(data_array[ind]):
                if ind == len(line_indicator) - 1:
                    reading = False
                    print('reading comlete! %d files, %d species, %d particles' %(len(data_array), species, global_part_num))
                    break
                else:
                    continue
            part_nums = np.array(re.split(r'\s+', data_array[ind][line_indicator[ind]].strip()), dtype=int)
            for s in range(species):
                for p in range(part_nums[s]):
                    line_indicator[ind] += 1
                    zpart = np.array(re.split(r'\s+', data_array[ind][line_indicator[ind]].strip()), dtype='float')
                    local_index = (int(zpart[-2]), int(zpart[-1]))
                    global_index = index_map[s][local_index]
                    orbit_array[global_index,:,step_indicator[global_index]] = zpart
                    step_indicator[global_index] += 1
            line_indicator[ind] += 1
    return orbit_array



def plot(orbit_array, global_index):
    
    fig1 = plt.figure()
    ax = fig1.add_subplot(2,2,1)
    ax.plot(orbit_array[global_index, 0, :])
    plt.title('R')
    ax = fig1.add_subplot(2,2,2)
    ax.plot(orbit_array[global_index, 1, :])
    plt.title('\zeta')
    ax = fig1.add_subplot(2,2,3)
    ax.plot(orbit_array[global_index, 2, :])
    plt.title('Z')
    energy = orbit_array[global_index, 3, :]**2+orbit_array[global_index, 5, :]**2*orbit_array[global_index, 10, :]**2+orbit_array[global_index, 7, :]**2
    ax = fig1.add_subplot(2,2,4)
    ax.plot(orbit_array[global_index,4,:])
    plt.title('energy')

    fig2 = plt.figure()
    ax = fig2.add_subplot(111, projection='3d')
    steps = len(orbit_array[global_index,0,:])
    for i in range(steps-1):
        ax.plot(orbit_array[global_index,0,i:i+2]*np.cos(orbit_array[global_index,1,i:i+2]),orbit_array[global_index,0,i:i+2]*np.sin(orbit_array[global_index,1,i:i+2]),orbit_array[global_index,2,i:i+2], color=plt.cm.jet(255*i//steps))

    ax.set(xlabel='x',ylabel='y',zlabel='z')
    ax.axis('equal')
    fig3 = plt.figure()
    ax = fig3.add_subplot(211)
    ax.plot(energy)
    ax.set_title('energy')
    ax = fig3.add_subplot(212)
    ax.plot(orbit_array[global_index,6,:])
    ax.set_title(r'$P_\zeta$')

    plt.ion()
    plt.show()

    
def plot2(orbit_array):
    fig= plt.figure()
    ax = fig.add_subplot(3,1,1)
    ax.plot(orbit_array[0,0,:])
    ax.plot(orbit_array[1,0,:])
    # ax.plot(orbit_array[2,0,:])
    # ax.plot(orbit_array[3,0,:])
    ax.set(ylabel='R')
    ax.legend(['FK','GK'])
    ax = fig.add_subplot(3,1,2)
    ax.plot(orbit_array[0,4,:])
    ax.plot(orbit_array[1,3,:])
    # ax.plot(orbit_array[2,3,:])
    # ax.plot(orbit_array[3,4,:])
    ax.set(ylabel='\zeta')
    ax.legend(['FK','GK'])
    ax = fig.add_subplot(3,1,3)
    ax.plot(orbit_array[0,2,:])
    ax.plot(orbit_array[1,2,:])
    # ax.plot(orbit_array[2,2,:])
    # ax.plot(orbit_array[3,2,:])
    ax.set(ylabel='Z')
    ax.set(xlabel='tstep/2')
    ax.legend(['FK','GK'])

    fig2 = plt.figure()
    ax = fig2.add_subplot(111,projection='3d')
    ax.plot(orbit_array[0,0,:]*np.cos(orbit_array[0,4,:]),orbit_array[0,0,:]*np.sin(orbit_array[0,4,:]),orbit_array[0,2,:])
    ax.plot(orbit_array[1,0,:]*np.cos(orbit_array[1,3,:]),orbit_array[1,0,:]*np.sin(orbit_array[1,3,:]),orbit_array[1,2,:])
    ax.set(xlabel='X',ylabel='Y',zlabel='Z')
    ax.axis('equal')
    ax.legend(['FK','GK'])

    plt.ion()
    plt.show()

def main(dirname):
    fname_array, data_array, mstep = pre_read(dirname)
    _, _, global_part_num, invert_map = index_mapping(data_array)
    orbit_array = read(fname_array, data_array, mstep)
    
    return orbit_array
if __name__ == "__main__":
   # plot2(orbit_array)
    dirname = os.path.join(os.path.abspath('.'),'trackp_dir')
    fname_array, data_array, mstep = pre_read(dirname)
    _, _, global_part_num, invert_map = index_mapping(data_array)
    orbit_array = read(fname_array, data_array, mstep)
    print('input particle index(0 - %d):' % (global_part_num-1))
    try:
        while True:
            if input == "":
                continue
            part_index = int(input())
            if(part_index >= global_part_num):
                print('illegal particle index, please reinput')
                continue
            plt.close('all')
            print("particle species: %d, local index: %d, initial pe: %d" %(invert_map[part_index]))
            plot(orbit_array, part_index)
            if part_index < 0:
                raise KeyboardInterrupt
    except KeyboardInterrupt as e:
        plt.close('all')
        print('\nExit...')
        raise KeyboardInterrupt
        

