import snap_read_f
import grids_read_f
import fluid_read_f
import numpy as np
import matplotlib
# matplotlib.use('pdf')
from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.tri as mtri
import argparse
import os
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from matplotlib.backend_bases import key_press_handler
from tkinter.filedialog import askopenfilename
from scipy import fftpack
import matplotlib.transforms as mtransforms


triang = None
fluid = False
uni_font = ("Helvetica", 20)
small_font = ("Helvetica", 14)
plt.rcParams.update({'font.size': 14, 'font.family': 'serif'})


class ToplevelData(tk.Toplevel):
    def __init__(self, deallocate_func, master=None, cnf={}, **kw):
        super(ToplevelData, self).__init__(master=master, cnf=cnf, **kw)
        self.deallocate = deallocate_func
        self.read = True

    def destroy(self):
        self.deallocate()
        if grids_read_f.grids_parameters.allocated:
            grids_read_f.grids_parameters.deallocate()
            grids_read_f.grids_parameters.allocated = False
        plt.close()
        super(ToplevelData, self).destroy()
        self.read = False


def mask_func(x, y, limit):
    distance0 = np.sqrt((x[:, 1]-x[:, 0])**2+(y[:, 1]-y[:, 0])**2)
    distance1 = np.sqrt((x[:, 2]-x[:, 1])**2+(y[:, 2]-y[:, 1])**2)
    distance2 = np.sqrt((x[:, 0]-x[:, 2])**2+(y[:, 0]-y[:, 2])**2)

    return np.max(np.array([distance0, distance1, distance2]), axis=0) > limit


def read(directory, snap_file_name, numerfrc):
    snap_read_f.snap_parameters.filename(snap_file_name)
    snap_read_f.snap_parameters.numerfrc = numerfrc
    snap_read_f.snap_parameters.snap_read()
    directory = os.path.dirname(snap_file_name)

    if not grids_read_f.grids_parameters.allocated:
        grids_file_name = os.path.join(directory, "grids.out")
        grids_read_f.grids_parameters.filename(grids_file_name)
        grids_read_f.grids_parameters.numerfrc = numerfrc
        grids_read_f.grids_parameters.grids_read()
        grids_read_f.grids_parameters.allocated = True

    print('Grids:'
          'mgrid_core:', grids_read_f.grids_parameters.mgrid_core,
          'mgrid_sol:', len(grids_read_f.grids_parameters.rsol))
    if numerfrc == 1:
        r_all = np.concatenate(
            (grids_read_f.grids_parameters.rcore, grids_read_f.grids_parameters.rsol))
        z_all = np.concatenate(
            (grids_read_f.grids_parameters.zcore, grids_read_f.grids_parameters.zsol))
    elif numerfrc == 3:
        r_all = grids_read_f.grids_parameters.rsol
        z_all = grids_read_f.grids_parameters.zsol
    global triang
    triang = mtri.Triangulation(r_all, z_all)
    print("mask out mesh lines shorter than 0.5")
    triang.set_mask(
        mask_func(r_all[triang.triangles], z_all[triang.triangles], 0.5))


def gen_2d_plot(window, title, values):

    new_win = tk.Toplevel(window)
    fm1 = tk.Frame(new_win)

    fig = plt.figure()
    ax = fig.add_subplot(111)
    # tpc = ax.tricontourf(triang, values, 20, cmap='jet')
    r_all = grids_read_f.grids_parameters.rsol
    z_all = grids_read_f.grids_parameters.zsol
    r_2d = r_all.reshape((grids_read_f.grids_parameters.mpsi_sol, -1))
    z_2d = z_all.reshape((grids_read_f.grids_parameters.mpsi_sol, -1))
    v_2d = values.reshape((grids_read_f.grids_parameters.mpsi_sol, -1))
    tpc = ax.pcolormesh(r_2d, z_2d, v_2d, cmap='jet', shading='gouraud')
    # tpc = ax.contourf(r_2d, z_2d, v_2d, 40, cmap='jet')

    ax.set_title(title)
    ax.set_xlabel("R", fontsize=14)
    ax.set_ylabel("Z", fontsize=14)
    cbar = fig.colorbar(tpc)
    cbar.formatter.set_powerlimits((-1, 1))
    cbar.update_ticks()
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()

    b_1d = tk.Button(fm1, text='1d plot',
                     command=lambda: plot1d(new_win, v_2d, r_2d, z_2d, title),
                     font=small_font)
    b_1d.pack(side='left', expand=False)

    rot = tk.BooleanVar()
    rot.set(False)
    b_rotate = tk.Button(fm1, text='rotate',
                         command=lambda: rotate(
                             ax, tpc, canvas, fig, rot),
                         font=small_font)
    b_rotate.pack(side="left", expand=False)
    equal = tk.BooleanVar()
    equal.set(False)
    b_change = tk.Button(fm1, text='aixs aspect Equal/Auto',
                         command=lambda: aspect(ax, canvas, fig, equal),
                         font=small_font)
    b_change.pack(side="left", expand=False)
    is_cbar = tk.BooleanVar()
    is_cbar.set(True)
    b_cbar = tk.Button(fm1, text='remove/add cbar',
                       command=lambda: switch_cbar(is_cbar, tpc, canvas, fig),
                       font=small_font)
    b_cbar.pack(side="left", expand=False)

    b_redraw = tk.Button(fm1, text='Redraw',
                         command=lambda: redraw(fig, canvas),
                         font=small_font)
    b_redraw.pack(side="left", expand=False)
    fm1.pack(side="top", fill="both", expand=True)

    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def plot1d(win, v_2d, r_2d, z_2d, name):
    winr = tk.Toplevel(win)
    figr = plt.figure()
    axr = figr.add_subplot(111)
    rgrids, zgrids = r_2d.shape
    axr.plot(r_2d[:, zgrids//2], v_2d[:, zgrids//2], '-o')
    axr.set_xlabel("R", fontsize=14)
    axr.set_ylabel(name, fontsize=14)
    figr.tight_layout()
    canvasr = FigureCanvasTkAgg(figr, master=winr)
    canvasr.draw()

    toolbarr = NavigationToolbar2Tk(canvasr, winr)
    toolbarr.update()
    canvasr.get_tk_widget().pack(side="top", fill='both', expand=True)

    winz = tk.Toplevel(win)
    figz = plt.figure()
    axz = figz.add_subplot(111)
    axz.plot(z_2d[rgrids//2, :], v_2d[rgrids//2, :], '-o')
    axz.set_xlabel("Z", fontsize=14)
    axz.set_ylabel(name, fontsize=14)
    canvasz = FigureCanvasTkAgg(figz, master=winz)
    canvasz.draw()

    toolbarz = NavigationToolbar2Tk(canvasz, winz)
    toolbarz.update()
    canvasz.get_tk_widget().pack(side="top", fill='both', expand=True)


def redraw(fig, canvas):
    fig.tight_layout()
    canvas.draw()


def switch_cbar(is_cbar, tpc, canvas, fig):
    ax = fig.axes[1]
    if is_cbar.get():
        ax.set_visible(False)
        is_cbar.set(False)
    else:
        ax.set_visible(True)
        is_cbar.set(True)
    fig.tight_layout()
    canvas.draw()


def rotate(ax, tpc, canvas, fig, rot):
    base = ax.transData
    rot_trans = mtransforms.Affine2D().rotate_deg(90)
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    xlabel = ax.get_xlabel()
    ylabel = ax.get_ylabel()
    if rot.get():
        tpc.set_transform(base)
        rot.set(False)
    else:
        tpc.set_transform(rot_trans+base)
        rot.set(True)
    ax.set_xlim(*ylim)
    ax.set_ylim(*xlim)
    ax.set_xlabel(ylabel)
    ax.set_ylabel(xlabel)
    fig.tight_layout()
    canvas.draw()


def aspect(ax, canvas, fig, equal):
    if equal.get():
        ax.set_aspect("auto")
        equal.set(False)
    else:
        ax.set_aspect("equal")
        equal.set(True)
    fig.tight_layout()
    canvas.draw()


def plot_grids(window):
    new_win = tk.Toplevel(window)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    tpc = ax.triplot(triang)
    ax.set_title('Grids')
    ax.set_xlabel("R", fontsize=14)
    ax.set_ylabel("Z", fontsize=14)

    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def plot_rz(window, title, values):
    r_arr = grids_read_f.grids_parameters.rg1d_sol
    # TODO: read n from gtc.out
    z_arr = np.linspace(
        0, 2*np.pi/20, snap_read_f.snap_parameters.mtoroidal, endpoint=False)
    Z, R = np.meshgrid(z_arr, r_arr)
    r_grid = r_arr.shape[0]
    z_grid = z_arr.shape[0]
    values_2d = values.reshape(r_grid, z_grid)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.set_title(title)
    tpc = ax.pcolormesh(Z, R, values_2d, shading='gouraud', cmap='jet')
    ax.set_xlabel("$\zeta$", fontsize=14)
    ax.set_ylabel("R", fontsize=14)
    cbar = fig.colorbar(tpc)
    new_win = tk.Toplevel(window)
    cbar.formatter.set_powerlimits((-1, 1))
    cbar.update_ticks()
    fig.tight_layout()
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def m_spect_sol(window, title, values):
    length = grids_read_f.grids_parameters.mpara_sol1
    spect_r = np.zeros((grids_read_f.grids_parameters.mpsi_sol, length))
    rrange = grids_read_f.grids_parameters.rg1d_sol
    values_2d = values.reshape(
        (grids_read_f.grids_parameters.mpsi_sol, length))
    for i in range(grids_read_f.grids_parameters.mpsi_sol):
        v_para = values_2d[i]
        spect = fftpack.fft(v_para)
        spect /= (length / 2)
        spect_r[i] = np.absolute(spect)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    legend = []
    for m in range(8):
        ax.plot(rrange, spect_r[:, m])
        legend.append('m=%d' % m)
    ax.legend(legend)
    ax.set_xlabel("R")
    ax.set_title(title)
    fig.tight_layout()
    new_win = tk.Toplevel(window)
    canvas = FigureCanvasTkAgg(fig, master=new_win)
    canvas.draw()

    toolbar = NavigationToolbar2Tk(canvas, new_win)
    toolbar.update()
    canvas.get_tk_widget().pack(side="top", fill='both', expand=True)


def panel(window, global_win_grp, work_dir):
    snap_file_name = askopenfilename(initialdir=work_dir, title="Select snapshot", filetypes=(
        ("GTC-X snapshots", "*.out"), ("all files", "*.*")))
    if not snap_file_name:
        return
    p = ToplevelData(snap_read_f.snap_parameters.deallocate,
                     window,  width=300, height=200)
    # p = tk.Toplevel(window, width=300, height=200)
    global_win_grp.append(p)
    p.title("snapshot")
    read(work_dir, snap_file_name, numerfrc=3)
    phi = snap_read_f.snap_parameters.phi
    b_phi = tk.Button(p, text='pol_phi', command=lambda: gen_2d_plot(
        window, "$\delta \phi$", phi), font=uni_font)
    b_phi.grid(row=1, column=1, sticky=tk.W+tk.E)

    phi_real = snap_read_f.snap_parameters.phi_real
    b_phi_real = tk.Button(p, text='pol_phi real', command=lambda: gen_2d_plot(
        window, "Real($\delta\phi$)", phi_real), font=uni_font)
    b_phi_real.grid(row=2, column=1, sticky=tk.W+tk.E)

    phi_imag = snap_read_f.snap_parameters.phi_imag
    b_phi_imag = tk.Button(p, text='pol_phi imag', command=lambda: gen_2d_plot(
        window, "Imag($\delta\phi$)", phi_imag), font=uni_font)
    b_phi_imag.grid(row=3, column=1, sticky=tk.W+tk.E)
    phi_cmplx = phi_real + 1j*phi_imag
    b_phi_spect = tk.Button(p, text='para modes', command=lambda: m_spect_sol(
        window, "$\delta \phi$", phi_cmplx), font=uni_font)
    b_phi_spect.grid(row=4, column=1, sticky=tk.W+tk.E)

    phi_rzeta_sol = snap_read_f.snap_parameters.phi_r_zeta_sol

    b_phi_rzeta_sol = tk.Button(p,
                                text='phi_rz sol',
                                command=lambda: plot_rz(
                                    window, r"SOL $\phi_{r\zeta}$", phi_rzeta_sol),
                                font=uni_font)
    b_phi_rzeta_sol.grid(row=5, column=1, sticky=tk.W+tk.E)

    phi_rzeta_core = snap_read_f.snap_parameters.phi_r_zeta_core
    b_phi_rzeta_core = tk.Button(p,
                                 text='phi_rz core',
                                 command=lambda: plot_rz(
                                     window, r"core $\phi_{r\zeta}$", phi_rzeta_core),
                                 font=uni_font,
                                 state="disable")
    b_phi_rzeta_core.grid(row=6, column=1, sticky=tk.W+tk.E)

    densityi = snap_read_f.snap_parameters.densityi
    b_densityi = tk.Button(p, text='pol_ni', command=lambda: gen_2d_plot(
        window, r"$\delta \bar{n}_i$", densityi), font=uni_font)
    b_densityi.grid(row=1, column=2, sticky=tk.W+tk.E)

    flowi = snap_read_f.snap_parameters.flowi
    b_flowi = tk.Button(p, text='pol_ui', command=lambda: gen_2d_plot(
        window, r"$\delta u_i$", flowi), font=uni_font)
    b_flowi.grid(row=2, column=2, sticky=tk.W+tk.E)

    pipara = snap_read_f.snap_parameters.pipara
    b_pipara = tk.Button(p, text='pi_para', command=lambda: gen_2d_plot(
        window, r"$\delta P_{i,\parallel}$", pipara), font=uni_font)
    b_pipara.grid(row=3, column=2, sticky=tk.W+tk.E)

    piperp = snap_read_f.snap_parameters.piperp
    b_piperp = tk.Button(p, text='pi_perp', command=lambda: gen_2d_plot(
        window, r"$\delta P_{i,\perp}$", piperp), font=uni_font)
    b_piperp.grid(row=4, column=2, sticky=tk.W+tk.E)

    ni_cmplx = densityi + 1j*0
    b_ni_spect = tk.Button(p, text='para modes', command=lambda: m_spect_sol(
        window, "$\delta n_i$", ni_cmplx), font=uni_font)
    b_ni_spect.grid(row=5, column=2, sticky=tk.W+tk.E)

    densitye = snap_read_f.snap_parameters.densitye
    b_densitye = tk.Button(p, text='pol_ne', command=lambda: gen_2d_plot(
        window, r"$\delta \bar{n}_e$", densitye), font=uni_font)
    b_densitye.grid(row=1, column=3, sticky=tk.W+tk.E)

    flowe = snap_read_f.snap_parameters.flowe
    b_flowe = tk.Button(p, text='pol_ui', command=lambda: gen_2d_plot(
        window, r"$\delta u_e$", flowe), font=uni_font)
    b_flowe.grid(row=2, column=3, sticky=tk.W+tk.E)

    pepara = snap_read_f.snap_parameters.pepara
    b_pepara = tk.Button(p, text='pe_para', command=lambda: gen_2d_plot(
        window, r"$\delta P_{e,\parallel}$", pepara), font=uni_font)
    b_pepara.grid(row=3, column=3, sticky=tk.W+tk.E)

    peperp = snap_read_f.snap_parameters.peperp
    b_peperp = tk.Button(p, text='pe_perp', command=lambda: gen_2d_plot(
        window, r"$\delta P_{e,\perp}$", peperp), font=uni_font)
    b_peperp.grid(row=4, column=3, sticky=tk.W+tk.E)

    ne_cmplx = densitye + 1j*0
    b_ne_spect = tk.Button(p, text='para modes', command=lambda: m_spect_sol(
        window, "$\delta n_e$", ne_cmplx), font=uni_font)
    b_ne_spect.grid(row=5, column=3, sticky=tk.W+tk.E)

    b_grids = tk.Button(p, text='Grids', command=lambda: plot_grids(
        window), font=uni_font)
    b_grids.grid(row=1, column=4, sticky=tk.W+tk.E)

    b_close = tk.Button(p, text='close', command=p.destroy, font=uni_font)
    b_close.grid(row=3, column=4, sticky=tk.W+tk.E)


def fluid_read(fluid_file_name):
    fluid_read_f.fluid_parameters.filename(fluid_file_name)
    directory = os.path.dirname(fluid_file_name)
    fluid_read_f.fluid_parameters.fluid_read()
    if not grids_read_f.grids_parameters.allocated:
        grids_file_name = os.path.join(directory, "grids.out")
        grids_read_f.grids_parameters.filename(grids_file_name)
        grids_read_f.grids_parameters.numerfrc = 3
        grids_read_f.grids_parameters.grids_read()
        grids_read_f.grids_parameters.allocated = True


def fluid_panel(window, global_win_grp, work_dir):
    fluid_file_name = askopenfilename(initialdir=work_dir, title="Select fluid snapshot", filetypes=(
        ("GTC-X snapshots", "*.out"), ("all files", "*.*")))
    if not fluid_file_name:
        return
    p = ToplevelData(fluid_read_f.fluid_parameters.deallocate,
                     window,  width=300, height=200)
    global_win_grp.append(p)
    p.title("Fluid snapshot")
    fluid_read(fluid_file_name)

    Efield = fluid_read_f.fluid_parameters.efield
    b_efield1 = tk.Button(p, text='ER', command=lambda: gen_2d_plot(
        window, r"$E_R$", Efield[0, :]), font=uni_font)
    b_efield1.grid(row=1, column=1, sticky=tk.W+tk.E)
    b_efield2 = tk.Button(p, text='Ezeta', command=lambda: gen_2d_plot(
        window, r"$E_\zeta$", Efield[1, :]), font=uni_font)
    b_efield2.grid(row=2, column=1, sticky=tk.W+tk.E)
    b_efield3 = tk.Button(p, text='Ez', command=lambda: gen_2d_plot(
        window, r"$E_z$", Efield[2, :]), font=uni_font)
    b_efield3.grid(row=3, column=1, sticky=tk.W+tk.E)
    
    curlE = fluid_read_f.fluid_parameters.curle
    b_curlE1 = tk.Button(p, text='curlER', command=lambda: gen_2d_plot(
       window, r"$(\nabla\times\mathbf{E})_R$", curlE[0, :]), font=uni_font)
    b_curlE2 = tk.Button(p, text='curlEzeta', command=lambda: gen_2d_plot(
       window, r"$(\nabla\times\mathbf{E})_\zeta$", curlE[1, :]), font=uni_font)
    b_curlE3 = tk.Button(p, text='curlEz', command=lambda: gen_2d_plot(
       window, r"$(\nabla\times\mathbf{E})_Z$", curlE[2, :]), font=uni_font)
    b_curlE1.grid(row=4,column=1,sticky=tk.W+tk.E)
    b_curlE2.grid(row=5,column=1,sticky=tk.W+tk.E)
    b_curlE3.grid(row=6,column=1,sticky=tk.W+tk.E)

    Bfield = fluid_read_f.fluid_parameters.bfield
    b_bfield1 = tk.Button(p, text='BR', command=lambda: gen_2d_plot(
        window, r"$B_R$", Bfield[0, :]), font=uni_font)
    b_bfield1.grid(row=1, column=2, sticky=tk.W+tk.E)
    b_bfield2 = tk.Button(p, text='Bzeta', command=lambda: gen_2d_plot(
        window, r"$B_\zeta$", Bfield[1, :]), font=uni_font)
    b_bfield2.grid(row=2, column=2, sticky=tk.W+tk.E)
    b_bfield3 = tk.Button(p, text='Bz', command=lambda: gen_2d_plot(
        window, r"$B_z$", Bfield[2, :]), font=uni_font)
    b_bfield3.grid(row=3, column=2, sticky=tk.W+tk.E)

    curlB = fluid_read_f.fluid_parameters.curlb
    b_curlB1 = tk.Button(p, text='curlBR', command=lambda: gen_2d_plot(
       window, r"$(\nabla\times\mathbf{B})_R$", curlB[0, :]), font=uni_font)
    b_curlB2 = tk.Button(p, text='curlBzeta', command=lambda: gen_2d_plot(
       window, r"$(\nabla\times\mathbf{B})_\zeta$", curlB[1, :]), font=uni_font)
    b_curlB3 = tk.Button(p, text='curlBz', command=lambda: gen_2d_plot(
       window, r"$(\nabla\times\mathbf{B})_Z$", curlB[2, :]), font=uni_font)
    b_curlB1.grid(row=4,column=2,sticky=tk.W+tk.E)
    b_curlB2.grid(row=5,column=2,sticky=tk.W+tk.E)
    b_curlB3.grid(row=6,column=2,sticky=tk.W+tk.E)


    ue = fluid_read_f.fluid_parameters.ue
    b_ue1 = tk.Button(p, text='ueR', command=lambda: gen_2d_plot(
        window, r"$ue_R$", ue[0, :]), font=uni_font)
    b_ue1.grid(row=1, column=3, sticky=tk.W+tk.E)
    b_ue2 = tk.Button(p, text='uezeta', command=lambda: gen_2d_plot(
        window, r"$ue_\zeta$", ue[1, :]), font=uni_font)
    b_ue2.grid(row=2, column=3, sticky=tk.W+tk.E)
    b_ue3 = tk.Button(p, text='uez', command=lambda: gen_2d_plot(
        window, r"$ue_z$", ue[2, :]), font=uni_font)
    b_ue3.grid(row=3, column=3, sticky=tk.W+tk.E)

    pe = fluid_read_f.fluid_parameters.pe
    b_pe = tk.Button(p, text='pe', command=lambda: gen_2d_plot(
        window, r"$P_e$", pe), font=uni_font)
    b_pe.grid(row=1, column=4, sticky=tk.W+tk.E)

    ne = fluid_read_f.fluid_parameters.ne
    b_ne = tk.Button(p, text='ne', command=lambda: gen_2d_plot(
        window, r"$n_e$", ne), font=uni_font)
    b_ne.grid(row=2, column=4, sticky=tk.W+tk.E)

    pene = fluid_read_f.fluid_parameters.pene
    b_pene = tk.Button(p, text='pene', command=lambda: gen_2d_plot(
        window, r"$P_en_e$", pene), font=uni_font)
    b_pene.grid(row=3, column=4, sticky=tk.W+tk.E)
